import { Router } from "express";
import { existsSync, readFileSync } from "fs";
import { ChatCompletionMessageParam, ChatCompletionSystemMessageParam } from "openai/resources";
import llm from "../services/llm";
import logger from "../utils/logger";
import { systemPromptPath, charactersPath } from "../utils/preload";
import path from "path";

const router = Router();

// 提取好感度的正则表达式
const affinityRegex = /^\|(\d+)\|/;

type ChatCompletionMessageWithAffinityParam = ChatCompletionMessageParam & {
  affinity: number;
}

router.post('/chat/:character', async (req, res) => {
  const { messages, stream = false } = req.body;
  const { character } = req.params;

  if (!character || !existsSync(path.resolve(charactersPath, `${character}.md`))) {
    res.status(404).json({ code: 404, message: '角色不存在' });
    return;
  }

  const characterPrompt = readFileSync(path.resolve(charactersPath, `${character}.md`), 'utf-8');

  let affinity = 0;

  logger.debug(`[${character}] 请求：${messages[messages.length - 1].content}`);

  const systemPrompt = readFileSync(systemPromptPath, 'utf-8');

  const systemMessage: ChatCompletionSystemMessageParam =
  {
    role: 'system',
    content: systemPrompt + '\n\n' + characterPrompt
  };

  const requestOptions = {
    messages: [systemMessage, ...messages.map((message: ChatCompletionMessageParam & ChatCompletionMessageWithAffinityParam) => {
      if (message.affinity) {
        return {
          role: message.role,
          content: `|${message.affinity}|${(message.content as string).replace(affinityRegex, '')}`
        }
      } else {
        return message;
      }
    })],
    model: "deepseek-chat",
  }

  try {
    if (stream) {
      // 流式响应
      res.setHeader('Content-Type', 'text/event-stream');
      res.setHeader('Cache-Control', 'no-cache');
      res.setHeader('Connection', 'keep-alive');

      const stream = await llm.chat.completions.create({
        ...requestOptions,
        stream: true,
      });

      let buffer = '';
      let totalContent = '';
      let affinityExtracted = false;

      for await (const chunk of stream) {
        const content = chunk.choices[0]?.delta?.content || '';
        buffer += content;

        // 处理缓冲区内容
        if (!affinityExtracted) {
          const match = affinityRegex.exec(buffer);

          if (match) {
            affinity = parseInt(match[1]);
            logger.debug(`[${character}] 好感度更新 ${affinity}`);
            res.write(`data: ${JSON.stringify({ affinity })}\n\n`);
            // 截取匹配后的剩余内容
            buffer = buffer.slice(match[0].length);
            affinityExtracted = true;
          }
        }

        // 如果已经提取过好感度，发送处理后的内容
        if (affinityExtracted) {
          // 发送当前缓冲区内容并清空
          if (buffer.length > 0) {
            res.write(`data: ${JSON.stringify({ content: buffer.trim() })}\n\n`);
            totalContent += buffer;
            buffer = '';
          }
        }
      }

      // 发送最后剩余的缓冲区内容
      if (buffer.length > 0) {
        res.write(`data: ${JSON.stringify({ content: buffer })}\n\n`);
        totalContent += buffer;
      }

      res.end();

      logger.debug(`[${character}] 回复：${totalContent}`);
    } else {
      // 普通响应
      const completion = await llm.chat.completions.create({
        ...requestOptions,
      });

      let content = completion.choices[0].message.content || '';
      const match = affinityRegex.exec(content);

      if (match) {
        affinity = parseInt(match[1]);
        logger.debug(`[${character}] 好感度更新 ${affinity}`);
        content = content.replace(affinityRegex, '').trim();
      }

      logger.debug(`[${character}] 回复：${content}`);

      res.json({ content, affinity });
    }
  } catch (error) {
    logger.error("生成文本时出错:", error);
    res.status(500).json({ code: 500, message: "生成文本时出错" });
  }
});

export default router;